'use strict';

let clock_running = false;
let clock_visible = false;
let comment_visible = false;
let lowerthird_visible = false;
let clock_elapsed = 0;
let clock_limit = 30 * 60;
let scoreA = 0;
let scoreB = 0;
let clock_origin;
let state = {};

function setteams()
{
	document.getElementById('team1').innerHTML = state['team1'];
	document.getElementById('team2').innerHTML = state['team2'];

	let sheets = { 'A': state['team1'], 'B': state['team2'] };
	loadquickl3s(sheets);
}

function setcolors()
{
	document.getElementById('team1color').style.backgroundColor = state['team1color'];
	document.getElementById('team2color').style.backgroundColor = state['team2color'];
}

function setscore()
{
	scoreA = state['score1'];
	scoreB = state['score2'];
	update_score();
}

function startclock()
{
	if (!clock_running) {
		clock_origin = Date.now();
		clock_running = true;
	}
	showclock();
}

function stopclock()
{
	if (!clock_running) return;
	clock_elapsed = time_elapsed();
	clock_origin = Date.now();
	clock_running = false;
}

function setclock(amount)
{
	clock_elapsed = amount;
	clock_origin = Date.now();
	update_clock();
}

function setclockfromstate()
{
	let amount = parseInt(state['clock_min']) * 60 + parseInt(state['clock_sec']);
	setclock(amount);
}

function setclocklimitfromstate()
{
	let amount = parseInt(state['clock_limit_min']) * 60 + parseInt(state['clock_limit_sec']);
	clock_limit = amount;
}

function showclock()
{
	if (clock_visible) return;
	let clockbug = document.getElementById('clockbug');
	clockbug.className = 'clockbug clockbug-animate-in';
	clock_visible = true;
}

function hideclock()
{
	if (!clock_visible) return;
	let clockbug = document.getElementById('clockbug');
	clockbug.className = 'clockbug clockbug-animate-out';
	clock_visible = false;
}

function setcomment()
{
	document.getElementById('comment').innerHTML = state['comment'];
}

function showcomment()
{
	if (comment_visible) return;
	let commentbug = document.getElementById('commentbug');
	commentbug.className = 'commentbug commentbug-animate-in';
	comment_visible = true;
}

function hidecomment()
{
	if (!comment_visible) return;
	let commentbug = document.getElementById('commentbug');
	commentbug.className = 'commentbug commentbug-animate-out';
	comment_visible = false;
}

function showlowerthird()
{
	if (lowerthird_visible) return;

	document.getElementById('lowerthird-headline').className = 'lowerthird-headline lowerthird-headline-animate-in';
	document.getElementById('lowerthird-headline-content').className = 'lowerthird-headline-content lowerthird-headline-content-animate-in';
	document.getElementById('lowerthird-subheading').className = 'lowerthird-subheading lowerthird-subheading-animate-in';
	document.getElementById('lowerthird-subheading-content').className = 'lowerthird-subheading-content lowerthird-subheading-content-animate-in';
	document.getElementById('lowerthird-picture').className = 'lowerthird-picture lowerthird-picture-animate-in';
	document.getElementById('lowerthird-picture-content').className = 'lowerthird-picture-content lowerthird-picture-content-animate-in';
	lowerthird_visible = true;
}

function setandshowlowerthird()
{
	document.getElementById('lowerthird-headline-content').innerHTML = state['text1'];
	document.getElementById('lowerthird-subheading-content').innerHTML = state['text2'];
	let img = document.getElementById('lowerthird-img');
	if (state['image'] === undefined) {
		img.style.display = 'none';
	} else {
		img.src = state['image'];	
		img.style.display = 'inline';
	}
	showlowerthird();
}

function hidelowerthird()
{
	if (!lowerthird_visible) return;
	document.getElementById('lowerthird-headline').className = 'lowerthird-headline lowerthird-headline-hidden lowerthird-headline-animate-out';
	document.getElementById('lowerthird-headline-content').className = 'lowerthird-headline-content lowerthird-headline-content-animate-out';
	document.getElementById('lowerthird-subheading').className = 'lowerthird-subheading lowerthird-subheading-animate-out';
	document.getElementById('lowerthird-subheading-content').className = 'lowerthird-subheading-content lowerthird-subheading-content-animate-out';
	document.getElementById('lowerthird-picture').className = 'lowerthird-picture lowerthird-picture-animate-out';
	document.getElementById('lowerthird-picture-content').className = 'lowerthird-picture-content lowerthird-picture-content-animate-out';
	lowerthird_visible = false;
}

function time_elapsed()
{
	let elapsed = (Date.now() - clock_origin) * 1e-3;
	if (clock_elapsed + elapsed >= clock_limit) {
		clock_elapsed = clock_limit;
		clock_origin = Date.now();
		clock_running = false;
		return clock_limit;
	}
	return Math.floor(clock_elapsed + elapsed);
}

function update_clock()
{
	let elapsed = time_elapsed();
	let min = Math.floor(elapsed / 60);
	let sec = elapsed % 60;

	if (sec < 10) sec = "0" + sec;
	document.getElementById('clock').innerHTML = min + ":" + sec;
}

function goalA()
{
	++scoreA;
	update_score();
}

function goalB()
{
	++scoreB;
	update_score();
}

function ungoalA()
{
	if (scoreA > 0) --scoreA;
	update_score();
}

function ungoalB()
{
	if (scoreB > 0) --scoreB;
	update_score();
}

function resetscore()
{
	scoreA = scoreB = 0;
	update_score();
}

function update_score()
{
	document.getElementById('score').innerHTML = scoreA + "&nbsp;–&nbsp;" + scoreB;
}

/* called by caspar only */
function play()
{
	document.getElementById('manualcontrols').style.display = 'none';
	document.getElementById('area').style.display = 'none';
}

function update(v)
{
	console.log('[[[' + v + ']]]');
	let j = JSON.parse(v);
	for(let key in j) state[key] = j[key];
}

setInterval(function() {
	if (clock_running) {
		update_clock();
	}
}, 100);
update_score();

//play();
//startclock();

let websocket = null;

function open_ws()
{
	console.log("Connecting...");
	try {
		if (websocket)
			websocket.close();
		websocket = new WebSocket("ws://127.0.0.1:5250/");
		websocket.onopen = function(evt) {
			console.log("Connected to client.");
		};
		websocket.onclose = function(evt) {
			console.log("Disconnected from client.");
			setTimeout(open_ws, 100);
		};
		websocket.onmessage = function(evt) {
			let msg = evt.data;
			let m = msg.match(/^update (.*)/);
			if (m !== null) {
				update(m[1]);
			}
			m = msg.match(/^eval (.*)/);
			if (m !== null) {
				eval(m[1]);
			}
		};
		websocket.onerror = function(evt) {
			console.log('Error: ' + evt.data);
		};
	} catch (exception) {
		console.log('Error: ' + exception);
		setTimeout(open_ws, 100);
	}
};
open_ws();
